@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif // HAVE_CONFIG_H
+#endif // defined(HAVE_CONFIG_H)
 
 #include <optional>
 
@@ -52,10 +52,6 @@ constexpr uint8_t H3_ALPN_V1[] = "\x2h3";
 // family from which packet is received.
 unsigned int msghdr_get_ecn(msghdr *msg, int family);
 
-// fd_set_ecn sets ECN bits |ecn| to |fd|.  |family| is the address
-// family of |fd|.
-void fd_set_ecn(int fd, int family, unsigned int ecn);
-
 // fd_set_recv_ecn sets socket option to |fd| so that it can receive
 // ECN bits.
 void fd_set_recv_ecn(int fd, int family);
@@ -87,4 +83,4 @@ bool addreq(const sockaddr *sa, const in_addr_union &iau);
 
 } // namespace ngtcp2
 
-#endif // SHARED_H
+#endif // !defined(SHARED_H)
