@@ -168,15 +168,15 @@ static int bbr_has_elapsed_in_phase(ngtcp2_cc_bbr *bbr,
 static uint64_t bbr_inflight_with_headroom(ngtcp2_cc_bbr *bbr,
                                            ngtcp2_conn_stat *cstat);
 
-static void bbr_raise_inflight_hi_slope(ngtcp2_cc_bbr *bbr,
-                                        ngtcp2_conn_stat *cstat);
+static void bbr_raise_inflight_longterm_slope(ngtcp2_cc_bbr *bbr,
+                                              ngtcp2_conn_stat *cstat);
 
-static void bbr_probe_inflight_hi_upward(ngtcp2_cc_bbr *bbr,
-                                         ngtcp2_conn_stat *cstat,
-                                         const ngtcp2_cc_ack *ack);
+static void bbr_probe_inflight_longterm_upward(ngtcp2_cc_bbr *bbr,
+                                               ngtcp2_conn_stat *cstat,
+                                               const ngtcp2_cc_ack *ack);
 
 static void bbr_adapt_upper_bounds(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
-                                   const ngtcp2_cc_ack *ack, ngtcp2_tstamp ts);
+                                   const ngtcp2_cc_ack *ack);
 
 static int bbr_is_time_to_probe_bw(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
                                    ngtcp2_tstamp ts);
@@ -189,24 +189,20 @@ static int bbr_is_reno_coexistence_probe_time(ngtcp2_cc_bbr *bbr,
 static uint64_t bbr_target_inflight(ngtcp2_cc_bbr *bbr,
                                     ngtcp2_conn_stat *cstat);
 
-static int bbr_check_inflight_too_high(ngtcp2_cc_bbr *bbr,
-                                       ngtcp2_conn_stat *cstat,
-                                       ngtcp2_tstamp ts);
-
-static int is_inflight_too_high(const ngtcp2_rs *rs);
+static int bbr_is_inflight_too_high(ngtcp2_cc_bbr *bbr);
 
 static void bbr_handle_inflight_too_high(ngtcp2_cc_bbr *bbr,
                                          ngtcp2_conn_stat *cstat,
-                                         const ngtcp2_rs *rs, ngtcp2_tstamp ts);
+                                         ngtcp2_tstamp ts);
 
 static void bbr_note_loss(ngtcp2_cc_bbr *bbr);
 
 static void bbr_handle_lost_packet(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
                                    const ngtcp2_cc_pkt *pkt, ngtcp2_tstamp ts);
 
-static uint64_t bbr_inflight_hi_from_lost_packet(ngtcp2_cc_bbr *bbr,
-                                                 const ngtcp2_rs *rs,
-                                                 const ngtcp2_cc_pkt *pkt);
+static uint64_t
+bbr_inflight_longterm_from_lost_packet(ngtcp2_cc_bbr *bbr,
+                                       const ngtcp2_cc_pkt *pkt);
 
 static void bbr_update_min_rtt(ngtcp2_cc_bbr *bbr, const ngtcp2_cc_ack *ack,
                                ngtcp2_tstamp ts);
@@ -326,7 +322,7 @@ static void bbr_on_init(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
   bbr->bw_probe_up_rounds = 0;
   bbr->bw_probe_up_acks = 0;
 
-  bbr->inflight_hi = UINT64_MAX;
+  bbr->inflight_longterm = UINT64_MAX;
 
   bbr->probe_rtt_expired = 0;
   bbr->probe_rtt_min_delay = UINT64_MAX;
@@ -349,8 +345,8 @@ static void bbr_reset_congestion_signals(ngtcp2_cc_bbr *bbr) {
 }
 
 static void bbr_reset_lower_bounds(ngtcp2_cc_bbr *bbr) {
-  bbr->bw_lo = UINT64_MAX;
-  bbr->inflight_lo = UINT64_MAX;
+  bbr->bw_shortterm = UINT64_MAX;
+  bbr->inflight_shortterm = UINT64_MAX;
 }
 
 static void bbr_init_round_counting(ngtcp2_cc_bbr *bbr) {
@@ -367,7 +363,7 @@ static void bbr_reset_full_bw(ngtcp2_cc_bbr *bbr) {
 
 static void bbr_check_full_bw_reached(ngtcp2_cc_bbr *bbr,
                                       ngtcp2_conn_stat *cstat) {
-  if (bbr->full_bw_now || bbr->rst->rs.is_app_limited) {
+  if (bbr->full_bw_now || !bbr->round_start || bbr->rst->rs.is_app_limited) {
     return;
   }
 
@@ -378,10 +374,6 @@ static void bbr_check_full_bw_reached(ngtcp2_cc_bbr *bbr,
     return;
   }
 
-  if (!bbr->round_start) {
-    return;
-  }
-
   ++bbr->full_bw_count;
 
   bbr->full_bw_now = bbr->full_bw_count >= 3;
@@ -399,13 +391,13 @@ static void bbr_check_startup_high_loss(ngtcp2_cc_bbr *bbr) {
   if (bbr->full_bw_reached || bbr->loss_events_in_round <= 6 ||
       (bbr->in_loss_recovery &&
        bbr->round_count <= bbr->round_count_at_recovery) ||
-      !is_inflight_too_high(&bbr->rst->rs)) {
+      !bbr_is_inflight_too_high(bbr)) {
     return;
   }
 
   bbr->full_bw_reached = 1;
-  bbr->inflight_hi = ngtcp2_max_uint64(bbr_bdp_multiple(bbr, bbr->cwnd_gain_h),
-                                       bbr->inflight_latest);
+  bbr->inflight_longterm = ngtcp2_max_uint64(
+    bbr_bdp_multiple(bbr, bbr->cwnd_gain_h), bbr->inflight_latest);
 }
 
 static void bbr_init_pacing_rate(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat) {
@@ -546,25 +538,26 @@ static void bbr_adapt_lower_bounds_from_congestion(ngtcp2_cc_bbr *bbr,
 }
 
 static void bbr_init_lower_bounds(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat) {
-  if (bbr->bw_lo == UINT64_MAX) {
-    bbr->bw_lo = bbr->max_bw;
+  if (bbr->bw_shortterm == UINT64_MAX) {
+    bbr->bw_shortterm = bbr->max_bw;
   }
 
-  if (bbr->inflight_lo == UINT64_MAX) {
-    bbr->inflight_lo = cstat->cwnd;
+  if (bbr->inflight_shortterm == UINT64_MAX) {
+    bbr->inflight_shortterm = cstat->cwnd;
   }
 }
 
 static void bbr_loss_lower_bounds(ngtcp2_cc_bbr *bbr) {
-  bbr->bw_lo = ngtcp2_max_uint64(
-    bbr->bw_latest, bbr->bw_lo * NGTCP2_BBR_BETA_NUMER / NGTCP2_BBR_BETA_DENOM);
-  bbr->inflight_lo = ngtcp2_max_uint64(
+  bbr->bw_shortterm = ngtcp2_max_uint64(
+    bbr->bw_latest,
+    bbr->bw_shortterm * NGTCP2_BBR_BETA_NUMER / NGTCP2_BBR_BETA_DENOM);
+  bbr->inflight_shortterm = ngtcp2_max_uint64(
     bbr->inflight_latest,
-    bbr->inflight_lo * NGTCP2_BBR_BETA_NUMER / NGTCP2_BBR_BETA_DENOM);
+    bbr->inflight_shortterm * NGTCP2_BBR_BETA_NUMER / NGTCP2_BBR_BETA_DENOM);
 }
 
 static void bbr_bound_bw_for_model(ngtcp2_cc_bbr *bbr) {
-  bbr->bw = ngtcp2_min_uint64(bbr->max_bw, bbr->bw_lo);
+  bbr->bw = ngtcp2_min_uint64(bbr->max_bw, bbr->bw_shortterm);
 }
 
 static void bbr_update_max_bw(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
@@ -738,7 +731,7 @@ static void bbr_start_probe_bw_up(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat) {
   bbr->pacing_gain_h = 125;
   bbr->cwnd_gain_h = 225;
 
-  bbr_raise_inflight_hi_slope(bbr, cstat);
+  bbr_raise_inflight_longterm_slope(bbr, cstat);
 }
 
 static void bbr_update_probe_bw_cycle_phase(ngtcp2_cc_bbr *bbr,
@@ -749,7 +742,7 @@ static void bbr_update_probe_bw_cycle_phase(ngtcp2_cc_bbr *bbr,
     return;
   }
 
-  bbr_adapt_upper_bounds(bbr, cstat, ack, ts);
+  bbr_adapt_upper_bounds(bbr, cstat, ack);
 
   if (!bbr_is_in_probe_bw_state(bbr)) {
     return;
@@ -806,7 +799,7 @@ static int bbr_is_time_to_cruise(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
 }
 
 static int bbr_is_time_to_go_down(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat) {
-  if (bbr->rst->is_cwnd_limited && cstat->cwnd >= bbr->inflight_hi) {
+  if (bbr->rst->is_cwnd_limited && cstat->cwnd >= bbr->inflight_longterm) {
     bbr_reset_full_bw(bbr);
     bbr->full_bw = cstat->delivery_rate_sec;
   } else if (bbr->full_bw_now) {
@@ -826,24 +819,25 @@ static uint64_t bbr_inflight_with_headroom(ngtcp2_cc_bbr *bbr,
                                            ngtcp2_conn_stat *cstat) {
   uint64_t headroom;
   uint64_t mpcwnd;
-  if (bbr->inflight_hi == UINT64_MAX) {
+  if (bbr->inflight_longterm == UINT64_MAX) {
     return UINT64_MAX;
   }
 
-  headroom = ngtcp2_max_uint64(cstat->max_tx_udp_payload_size,
-                               bbr->inflight_hi * NGTCP2_BBR_HEADROOM_NUMER /
-                                 NGTCP2_BBR_HEADROOM_DENOM);
+  headroom =
+    ngtcp2_max_uint64(cstat->max_tx_udp_payload_size,
+                      bbr->inflight_longterm * NGTCP2_BBR_HEADROOM_NUMER /
+                        NGTCP2_BBR_HEADROOM_DENOM);
   mpcwnd = min_pipe_cwnd(cstat->max_tx_udp_payload_size);
 
-  if (bbr->inflight_hi > headroom) {
-    return ngtcp2_max_uint64(bbr->inflight_hi - headroom, mpcwnd);
+  if (bbr->inflight_longterm > headroom) {
+    return ngtcp2_max_uint64(bbr->inflight_longterm - headroom, mpcwnd);
   }
 
   return mpcwnd;
 }
 
-static void bbr_raise_inflight_hi_slope(ngtcp2_cc_bbr *bbr,
-                                        ngtcp2_conn_stat *cstat) {
+static void bbr_raise_inflight_longterm_slope(ngtcp2_cc_bbr *bbr,
+                                              ngtcp2_conn_stat *cstat) {
   uint64_t growth_this_round = cstat->max_tx_udp_payload_size
                                << bbr->bw_probe_up_rounds;
 
@@ -851,12 +845,12 @@ static void bbr_raise_inflight_hi_slope(ngtcp2_cc_bbr *bbr,
   bbr->probe_up_cnt = ngtcp2_max_uint64(cstat->cwnd / growth_this_round, 1);
 }
 
-static void bbr_probe_inflight_hi_upward(ngtcp2_cc_bbr *bbr,
-                                         ngtcp2_conn_stat *cstat,
-                                         const ngtcp2_cc_ack *ack) {
+static void bbr_probe_inflight_longterm_upward(ngtcp2_cc_bbr *bbr,
+                                               ngtcp2_conn_stat *cstat,
+                                               const ngtcp2_cc_ack *ack) {
   uint64_t delta;
 
-  if (!bbr->rst->is_cwnd_limited || cstat->cwnd < bbr->inflight_hi) {
+  if (!bbr->rst->is_cwnd_limited || cstat->cwnd < bbr->inflight_longterm) {
     return;
   }
 
@@ -867,16 +861,16 @@ static void bbr_probe_inflight_hi_upward(ngtcp2_cc_bbr *bbr,
         bbr->probe_up_cnt * cstat->max_tx_udp_payload_size) {
     delta = bbr->bw_probe_up_acks / bbr->probe_up_cnt;
     bbr->bw_probe_up_acks -= delta * bbr->probe_up_cnt;
-    bbr->inflight_hi += delta;
+    bbr->inflight_longterm += delta;
   }
 
   if (bbr->round_start) {
-    bbr_raise_inflight_hi_slope(bbr, cstat);
+    bbr_raise_inflight_longterm_slope(bbr, cstat);
   }
 }
 
 static void bbr_adapt_upper_bounds(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
-                                   const ngtcp2_cc_ack *ack, ngtcp2_tstamp ts) {
+                                   const ngtcp2_cc_ack *ack) {
   if (bbr->ack_phase == NGTCP2_BBR_ACK_PHASE_ACKS_PROBE_STARTING &&
       bbr->round_start) {
     bbr->ack_phase = NGTCP2_BBR_ACK_PHASE_ACKS_PROBE_FEEDBACK;
@@ -889,17 +883,17 @@ static void bbr_adapt_upper_bounds(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
     }
   }
 
-  if (!bbr_check_inflight_too_high(bbr, cstat, ts)) {
-    if (bbr->inflight_hi == UINT64_MAX) {
+  if (!bbr_is_inflight_too_high(bbr)) {
+    if (bbr->inflight_longterm == UINT64_MAX) {
       return;
     }
 
-    if (bbr->rst->rs.tx_in_flight > bbr->inflight_hi) {
-      bbr->inflight_hi = bbr->rst->rs.tx_in_flight;
+    if (bbr->rst->rs.tx_in_flight > bbr->inflight_longterm) {
+      bbr->inflight_longterm = bbr->rst->rs.tx_in_flight;
     }
 
     if (bbr->state == NGTCP2_BBR_STATE_PROBE_BW_UP) {
-      bbr_probe_inflight_hi_upward(bbr, cstat, ack);
+      bbr_probe_inflight_longterm_upward(bbr, cstat, ack);
     }
   }
 }
@@ -941,33 +935,21 @@ static uint64_t bbr_target_inflight(ngtcp2_cc_bbr *bbr,
   return ngtcp2_min_uint64(bbr->bdp, cstat->cwnd);
 }
 
-static int bbr_check_inflight_too_high(ngtcp2_cc_bbr *bbr,
-                                       ngtcp2_conn_stat *cstat,
-                                       ngtcp2_tstamp ts) {
-  if (is_inflight_too_high(&bbr->rst->rs)) {
-    if (bbr->bw_probe_samples) {
-      bbr_handle_inflight_too_high(bbr, cstat, &bbr->rst->rs, ts);
-    }
-
-    return 1;
-  }
-
-  return 0;
-}
-
-static int is_inflight_too_high(const ngtcp2_rs *rs) {
+static int bbr_is_inflight_too_high(ngtcp2_cc_bbr *bbr) {
+  const ngtcp2_rs *rs = &bbr->rst->rs;
   return rs->lost * NGTCP2_BBR_LOSS_THRESH_DENOM >
          rs->tx_in_flight * NGTCP2_BBR_LOSS_THRESH_NUMER;
 }
 
 static void bbr_handle_inflight_too_high(ngtcp2_cc_bbr *bbr,
                                          ngtcp2_conn_stat *cstat,
-                                         const ngtcp2_rs *rs,
                                          ngtcp2_tstamp ts) {
+  const ngtcp2_rs *rs = &bbr->rst->rs;
+
   bbr->bw_probe_samples = 0;
 
   if (!rs->is_app_limited) {
-    bbr->inflight_hi = ngtcp2_max_uint64(
+    bbr->inflight_longterm = ngtcp2_max_uint64(
       rs->tx_in_flight, bbr_target_inflight(bbr, cstat) *
                           NGTCP2_BBR_BETA_NUMER / NGTCP2_BBR_BETA_DENOM);
   }
@@ -987,7 +969,7 @@ static void bbr_note_loss(ngtcp2_cc_bbr *bbr) {
 
 static void bbr_handle_lost_packet(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
                                    const ngtcp2_cc_pkt *pkt, ngtcp2_tstamp ts) {
-  ngtcp2_rs rs = {0};
+  ngtcp2_rs *rs = &bbr->rst->rs;
 
   bbr_note_loss(bbr);
 
@@ -995,22 +977,23 @@ static void bbr_handle_lost_packet(ngtcp2_cc_bbr *bbr, ngtcp2_conn_stat *cstat,
     return;
   }
 
-  rs.tx_in_flight = pkt->tx_in_flight;
+  rs->tx_in_flight = pkt->tx_in_flight;
   /* bbr->rst->lost is not incremented for pkt yet */
   assert(bbr->rst->lost + pkt->pktlen >= pkt->lost);
-  rs.lost = bbr->rst->lost + pkt->pktlen - pkt->lost;
-  rs.is_app_limited = pkt->is_app_limited;
+  rs->lost = bbr->rst->lost + pkt->pktlen - pkt->lost;
+  rs->is_app_limited = pkt->is_app_limited;
 
-  if (is_inflight_too_high(&rs)) {
-    rs.tx_in_flight = bbr_inflight_hi_from_lost_packet(bbr, &rs, pkt);
+  if (bbr_is_inflight_too_high(bbr)) {
+    rs->tx_in_flight = bbr_inflight_longterm_from_lost_packet(bbr, pkt);
 
-    bbr_handle_inflight_too_high(bbr, cstat, &rs, ts);
+    bbr_handle_inflight_too_high(bbr, cstat, ts);
   }
 }
 
-static uint64_t bbr_inflight_hi_from_lost_packet(ngtcp2_cc_bbr *bbr,
-                                                 const ngtcp2_rs *rs,
-                                                 const ngtcp2_cc_pkt *pkt) {
+static uint64_t
+bbr_inflight_longterm_from_lost_packet(ngtcp2_cc_bbr *bbr,
+                                       const ngtcp2_cc_pkt *pkt) {
+  ngtcp2_rs *rs = &bbr->rst->rs;
   uint64_t inflight_prev, lost_prev, lost_prefix;
   (void)bbr;
 
@@ -1204,9 +1187,6 @@ static void bbr_update_max_inflight(ngtcp2_cc_bbr *bbr,
                                     ngtcp2_conn_stat *cstat) {
   uint64_t inflight;
 
-  /* Not documented */
-  /* bbr_update_aggregation_budget(bbr); */
-
   inflight = bbr_bdp_multiple(bbr, bbr->cwnd_gain_h) + bbr->extra_acked;
   bbr->max_inflight = bbr_quantization_budget(bbr, cstat, inflight);
 }
@@ -1281,13 +1261,13 @@ static void bbr_bound_cwnd_for_model(ngtcp2_cc_bbr *bbr,
 
   if (bbr_is_in_probe_bw_state(bbr) &&
       bbr->state != NGTCP2_BBR_STATE_PROBE_BW_CRUISE) {
-    cap = bbr->inflight_hi;
+    cap = bbr->inflight_longterm;
   } else if (bbr->state == NGTCP2_BBR_STATE_PROBE_RTT ||
              bbr->state == NGTCP2_BBR_STATE_PROBE_BW_CRUISE) {
     cap = bbr_inflight_with_headroom(bbr, cstat);
   }
 
-  cap = ngtcp2_min_uint64(cap, bbr->inflight_lo);
+  cap = ngtcp2_min_uint64(cap, bbr->inflight_shortterm);
   cap = ngtcp2_max_uint64(cap, mpcwnd);
 
   cstat->cwnd = ngtcp2_min_uint64(cstat->cwnd, cap);
