@@ -34,6 +34,7 @@
 
 static const MunitTest tests[] = {
   munit_void_test(test_ngtcp2_transport_params_encode),
+  munit_void_test(test_ngtcp2_transport_params_decode),
   munit_void_test(test_ngtcp2_transport_params_decode_new),
   munit_void_test(test_ngtcp2_transport_params_convert_to_latest),
   munit_void_test(test_ngtcp2_transport_params_convert_to_old),
@@ -41,7 +42,8 @@ static const MunitTest tests[] = {
 };
 
 const MunitSuite transport_params_suite = {
-  "/transport_params", tests, NULL, 1, MUNIT_SUITE_OPTION_NONE,
+  .prefix = "/transport_params",
+  .tests = tests,
 };
 
 static size_t varint_paramlen(ngtcp2_transport_param_id id, uint64_t value) {
@@ -50,7 +52,7 @@ static size_t varint_paramlen(ngtcp2_transport_param_id id, uint64_t value) {
 }
 
 void test_ngtcp2_transport_params_encode(void) {
-  ngtcp2_transport_params params, nparams;
+  ngtcp2_transport_params params = {0}, nparams = {0};
   uint8_t buf[512];
   ngtcp2_ssize nwrite;
   int rv;
@@ -63,9 +65,6 @@ void test_ngtcp2_transport_params_encode(void) {
   scid_init(&scid);
   dcid_init(&dcid);
 
-  memset(&params, 0, sizeof(params));
-  memset(&nparams, 0, sizeof(nparams));
-
   for (i = 0; i < sizeof(available_versions); i += sizeof(uint32_t)) {
     ngtcp2_put_uint32be(&available_versions[i], (uint32_t)(0xff000000u + i));
   }
@@ -177,7 +176,15 @@ void test_ngtcp2_transport_params_encode(void) {
 
   assert_ptrdiff((ngtcp2_ssize)i, ==, nwrite);
 
-  rv = ngtcp2_transport_params_decode(&nparams, buf, (size_t)nwrite);
+  for (i = 0; i < len; ++i) {
+    rv = ngtcp2_transport_params_decode(&nparams, buf, i);
+
+    assert(0 == rv || NGTCP2_ERR_MALFORMED_TRANSPORT_PARAM == rv);
+  }
+
+  rv = ngtcp2_transport_params_decode(&nparams, buf, len);
+
+  assert_int(0, ==, rv);
 
   assert_int(0, ==, rv);
   assert_uint64(params.initial_max_stream_data_bidi_local, ==,
@@ -236,8 +243,41 @@ void test_ngtcp2_transport_params_encode(void) {
                       nparams.version_info.available_versions);
 }
 
+void test_ngtcp2_transport_params_decode(void) {
+  ngtcp2_transport_params params;
+  int rv;
+
+  /* Decode from 0 length data */
+  rv = ngtcp2_transport_params_decode(&params, NULL, 0);
+
+  assert_int(0, ==, rv);
+  assert_uint64(0, ==, params.initial_max_stream_data_bidi_local);
+  assert_uint64(0, ==, params.initial_max_stream_data_bidi_remote);
+  assert_uint64(0, ==, params.initial_max_stream_data_uni);
+  assert_uint64(0, ==, params.initial_max_data);
+  assert_uint64(0, ==, params.initial_max_streams_bidi);
+  assert_uint64(0, ==, params.initial_max_streams_uni);
+  assert_uint64(0, ==, params.max_idle_timeout);
+  assert_uint64(NGTCP2_DEFAULT_MAX_RECV_UDP_PAYLOAD_SIZE, ==,
+                params.max_udp_payload_size);
+  assert_false(params.stateless_reset_token_present);
+  assert_uint64(NGTCP2_DEFAULT_ACK_DELAY_EXPONENT, ==,
+                params.ack_delay_exponent);
+  assert_false(params.preferred_addr_present);
+  assert_false(params.disable_active_migration);
+  assert_uint64(NGTCP2_DEFAULT_MAX_ACK_DELAY, ==, params.max_ack_delay);
+  assert_false(params.retry_scid_present);
+  assert_false(params.initial_scid_present);
+  assert_false(params.original_dcid_present);
+  assert_uint64(NGTCP2_DEFAULT_ACTIVE_CONNECTION_ID_LIMIT, ==,
+                params.active_connection_id_limit);
+  assert_uint64(0, ==, params.max_datagram_frame_size);
+  assert_false(params.grease_quic_bit);
+  assert_false(params.version_info_present);
+}
+
 void test_ngtcp2_transport_params_decode_new(void) {
-  ngtcp2_transport_params params, *nparams;
+  ngtcp2_transport_params params = {0}, *nparams;
   uint8_t buf[512];
   ngtcp2_ssize nwrite;
   int rv;
@@ -250,9 +290,6 @@ void test_ngtcp2_transport_params_decode_new(void) {
   scid_init(&scid);
   dcid_init(&dcid);
 
-  memset(&params, 0, sizeof(params));
-  memset(&nparams, 0, sizeof(nparams));
-
   for (i = 0; i < sizeof(available_versions); i += sizeof(uint32_t)) {
     ngtcp2_put_uint32be(&available_versions[i], (uint32_t)(0xff000000u + i));
   }
@@ -355,7 +392,18 @@ void test_ngtcp2_transport_params_decode_new(void) {
 
   assert_ptrdiff((ngtcp2_ssize)len, ==, nwrite);
 
-  rv = ngtcp2_transport_params_decode_new(&nparams, buf, (size_t)nwrite, NULL);
+  for (i = 0; i < len; ++i) {
+    rv = ngtcp2_transport_params_decode_new(&nparams, buf, i, NULL);
+
+    if (rv != 0) {
+      assert_int(NGTCP2_ERR_MALFORMED_TRANSPORT_PARAM, ==, rv);
+      continue;
+    }
+
+    ngtcp2_transport_params_del(nparams, NULL);
+  }
+
+  rv = ngtcp2_transport_params_decode_new(&nparams, buf, len, NULL);
 
   assert_int(0, ==, rv);
   assert_uint64(params.initial_max_stream_data_bidi_local, ==,
@@ -371,6 +419,7 @@ void test_ngtcp2_transport_params_decode_new(void) {
                 nparams->initial_max_streams_uni);
   assert_uint64(params.max_idle_timeout, ==, nparams->max_idle_timeout);
   assert_uint64(params.max_udp_payload_size, ==, nparams->max_udp_payload_size);
+  assert_true(nparams->stateless_reset_token_present);
   assert_memory_equal(sizeof(params.stateless_reset_token),
                       params.stateless_reset_token,
                       nparams->stateless_reset_token);
@@ -424,11 +473,16 @@ void test_ngtcp2_transport_params_convert_to_latest(void) {
   ngtcp2_cid rcid, scid, dcid;
   uint8_t available_versions[sizeof(uint32_t) * 3];
   ngtcp2_sockaddr_in6 *sa_in6;
+  size_t i;
 
   rcid_init(&rcid);
   scid_init(&scid);
   dcid_init(&dcid);
 
+  for (i = 0; i < sizeof(available_versions); i += sizeof(uint32_t)) {
+    ngtcp2_put_uint32be(&available_versions[i], (uint32_t)(0xff000000u + i));
+  }
+
   ngtcp2_transport_params_default_versioned(NGTCP2_TRANSPORT_PARAMS_V1,
                                             &srcbuf);
 
